@@ -77,8 +77,13 @@ public:
     memset(this, 0, sizeof(LASattribute));
     scale[0] = scale[1] = scale[2] = 1.0;
     this->data_type = (dim-1)*10+type+1;
-    strncpy(this->name, name, 31);
-    if (description) strncpy(this->description, description, 31);
+    int len = 0 ; while(*(name+len) != '\0' && len < 32) len++;
+    memcpy(this->name, name, len);
+    if (description)
+    {
+      int len = 0 ; while(*(description+len) != '\0' && len < 32) len++;
+      memcpy(this->description, description, len);
+    }
   };
 
   inline BOOL set_no_data(U8 no_data, I32 dim=0) { if ((0 == get_type()) && (dim < get_dim())) { this->no_data[dim].u64 = no_data; options |= 0x01; return TRUE; } return FALSE; };
